@@ -1,9 +1,9 @@
 /**
  *
- * @fileOverview Chooser (vm list) singleton
+ * @fileOverview Chooser (vm list) singleton. Provides vboxChooser
  * @author Ian Moore (imoore76 at yahoo dot com)
- * @version $Id: chooser.js 496 2012-11-02 13:04:44Z imooreyahoo@gmail.com $
- * @copyright Copyright (C) 2010-2012 Ian Moore (imoore76 at yahoo dot com)
+ * @version $Id: chooser.js 591 2015-04-11 22:40:47Z imoore76 $
+ * @copyright Copyright (C) 2010-2015 Ian Moore (imoore76 at yahoo dot com)
  *
  */
 
@@ -28,7 +28,7 @@ var vboxChooser = {
 	vms : {},
 	
 	// VM tool tip
-	_vmToolTip : trans('<nobr>%1<br></nobr><nobr>%2 since %3</nobr><br><nobr>Session %4</nobr>','UIVMListView'),
+	_vmToolTip : '<nobr>%1<br></nobr><nobr>%2 since %3</nobr><br><nobr>Session %4</nobr>',
 	
 	// Anchor element
 	_anchorid : null,
@@ -57,6 +57,11 @@ var vboxChooser = {
 	 */
 	_versionChecked : false,
 	
+	/* Some items are not editable while vmGroup
+	 * definitions are being written
+	 */
+	_editable : true,
+	
 	/* Context menus */
 	_vmContextMenuObj : null,
 	_vmGroupContextMenuObj : null,
@@ -67,6 +72,9 @@ var vboxChooser = {
 	/* Group definition extra value key */
 	_groupDefinitionKey : '',
 	
+	/* Whether chooser is in compact mode or not */
+	_compact : false,
+	
 	/**
 	 * Set anchor id to draw to
 	 */
@@ -83,7 +91,19 @@ var vboxChooser = {
 		});
 		
 		$(window).resize(function(){
+			
+			// Get anchor id and add / remove class
+			var w = parseInt($(vboxChooser._anchor).innerWidth());
+			if(w < 120) {
+				$(vboxChooser._anchor).addClass('vboxChooserMini');
+				vboxChooser._compact = true;
+			} else {
+				$(vboxChooser._anchor).removeClass('vboxChooserMini');
+				vboxChooser._compact = false;
+			}
+
 			vboxChooser._resizeElements(true);
+			
 		});
 	},
 	
@@ -97,20 +117,26 @@ var vboxChooser = {
 		
 			// Group menu
 			case 'group':
-				vboxChooser._vmGroupContextMenuObj = new vboxMenu(vboxChooser._anchorid+'vmgroups',null,menuitems);
+				vboxChooser._vmGroupContextMenuObj = new vboxMenu({'name': vboxChooser._anchorid+'vmgroups',
+				                                                   'menuItems': menuitems,
+				                                                   'language_context': 'UIActionPool'});
 				vboxChooser._vmGroupContextMenuObj.update();
 				break;
 				
 			// VM Menu
 			case 'vm':
-				vboxChooser._vmContextMenuObj = new vboxMenu(vboxChooser._anchorid+'vms',null,menuitems);
+				vboxChooser._vmContextMenuObj = new vboxMenu({'name': vboxChooser._anchorid+'vms',
+				                                              'menuItems': menuitems,
+				                                              'language_context': 'UIActionPool'});
 				vboxChooser._vmContextMenuObj.update();
 				break;
 				
 			// Main list menu
 			case 'anchor':
 
-				var vboxChooserPaneMenu = new vboxMenu(vboxChooser._anchorid+'Pane',null,menuitems);			
+				var vboxChooserPaneMenu = new vboxMenu({'name': vboxChooser._anchorid+'Pane',
+				                                        'menuItems': menuitems,
+				                                        'language_context': 'UIActionPool'});			
 				$('#'+vboxChooser._anchorid).parent().contextMenu({
 				  		menu: vboxChooserPaneMenu.menuId()
 				  	},
@@ -278,10 +304,10 @@ var vboxChooser = {
 		
 		// Render root group
 		vboxChooser._anchor.append(vboxChooser.groupHTML("/"));
-		
+				
 		// Enforce VM ownership
         if($('#vboxPane').data('vboxConfig').enforceVMOwnership && !$('#vboxPane').data('vboxSession').admin) {
-        	d.vmlist = jQuery.grep(vmlist,function(vm,i){
+        	vmlist = jQuery.grep(vmlist,function(vm,i){
         		return (vm.owner == $('#vboxPane').data('vboxSession').user);
         	});
 		}
@@ -360,10 +386,10 @@ var vboxChooser = {
 		
 		vboxChooser._scrollbarWasVisible = sbVisible;
 		
-		var groupTitleWidth = vboxChooser._anchor.width() - 32 - (sbVisible ? vboxChooser._scrollbarWidth : 0);
-		var vmTitleWidth = groupTitleWidth - 18; // (2px padding on .vboxChooserGroupVMs + 
+		var groupTitleWidth = vboxChooser._anchor.width() - (vboxChooser._compact ? 22 : 32) - (sbVisible ? vboxChooser._scrollbarWidth : 0);
+		var vmTitleWidth = groupTitleWidth - (vboxChooser._compact ? -12 : 18); // (2px padding on .vboxChooserGroupVMs + 
 			// 2px border on table + 4px margin on icon) * 2 
-		var groupLevelOffset = 8; // (2px margin + 2px border) * 2
+		var groupLevelOffset = (vboxChooser._compact ? 8 : 8); // (2px margin + 2px border) * 2
 		
 		
 		// Now that we have sizes, we can inject styles
@@ -373,9 +399,11 @@ var vboxChooser = {
 		var path = ['div.vboxChooserGroupRootLevel'];
 		
 		// Special case for root level VM list
-		styleRules[styleRules.length] = 'div.vboxChooserGroupRootLevel > div.vboxChooserGroupVMs td.vboxVMTitle > div { width: ' + vmTitleWidth + 'px; }';
+		styleRules[styleRules.length] = 'div.vboxChooserGroupRootLevel > div.vboxChooserGroupVMs table.vboxChooserVM div.vboxFitToContainer { width: ' + vmTitleWidth + 'px; }';
+		
 		// Special case for group header when only showing one group
 		styleRules[styleRules.length] = 'div.vboxChooserGroupShowOnly.vboxChooserGroupRootLevel > div.vboxChooserGroupHeader span.vboxChooserGroupName { max-width: ' + (groupTitleWidth - 4) + 'px; }';
+		
 		// Bottom group resize bars
 		styleRules[styleRules.length] = 'div.vboxChooserGroupRootLevel > div.vboxChooserDropTargetBottom { width: ' + (groupTitleWidth) + 30 + 'px; }';
 
@@ -385,17 +413,37 @@ var vboxChooser = {
 			styleRules[styleRules.length] = path.join(' > ') + ' > div.vboxChooserGroup > div.vboxChooserGroupHeader span.vboxChooserGroupName { max-width: ' + (groupTitleWidth - (i*groupLevelOffset)) + 'px; }';
 			
 			// VM titles at this level
-			styleRules[styleRules.length] = path.join(' > ') + ' > div.vboxChooserGroup > div.vboxChooserGroupVMs td.vboxVMTitle > div { width: ' + (vmTitleWidth - (i*(groupLevelOffset))) + 'px; }';
+			styleRules[styleRules.length] = path.join(' > ') + ' > div.vboxChooserGroup > div.vboxChooserGroupVMs table.vboxChooserVM div.vboxFitToContainer { width: ' + (vmTitleWidth - (i*(groupLevelOffset))) + 'px; }';
 
 			// Bottom group resize bars
 			styleRules[styleRules.length] = path.join(' > ') +' > div.vboxChooserGroup > div.vboxChooserDropTargetBottom { width: ' + (groupTitleWidth + 30 - (i*groupLevelOffset)) + 'px; }';
 
 			path[path.length] = 'div.vboxChooserGroup';
 		}
-		$('head').append('<style type="text/css" id="vboxChooserStyle">' + styleRules.join("\n") + '</style>');
+	
+		// Style for minified vmlist
+		if(vboxChooser._compact) {
+			// Title moves left
+			styleRules[styleRules.length] = 'div.vboxChooserGroup > div.vboxChooserGroupVMs table.vboxChooserVM div.vboxVMName { position: relative; left: -20px; }';
+			// Icon moves down
+			styleRules[styleRules.length] = 'div.vboxChooserGroup > div.vboxChooserGroupVMs table.vboxChooserVM img.vboxVMIcon { position: relative; top: 8px; }';
+			// State text goes away
+			styleRules[styleRules.length] = 'div.vboxChooserGroup > div.vboxChooserGroupVMs table.vboxChooserVM span.vboxVMState { display: none; }';
+			// Less padding
+			styleRules[styleRules.length] = 'div.vboxChooserGroup > div.vboxChooserGroupVMs table.vboxChooserVM td { padding: 0px; }';
+			// Some group header items and drop targets go away
+			styleRules[styleRules.length] = 'div.vboxChooserGroup > div.vboxChooserGroupHeader > .vboxChooserGroupNameArrowCollapse, #' +vboxChooser._anchorid + ' div.vboxChooserGroup .vboxChooserDropTarget { display: none; }';
+			styleRules[styleRules.length] = 'div.vboxChooserGroup { overflow: hidden; }';
+			// host
+			styleRules[styleRules.length] = '#vboxChooserVMHost .vboxVMState { display: none; }';
+			// group header 
+			styleRules[styleRules.length] = 'div.vboxChooserGroup div.vboxChooserGroupHeader { height: auto; padding: 2px; }';
+
+		}
+		$('head').append('<style type="text/css" id="vboxChooserStyle">#'+vboxChooser._anchorid + ' ' + styleRules.join("\n#"+vboxChooser._anchorid + " ") + '</style>');
 		
 	},
-	
+		
 	/*
 	 * Get group element by path
 	 */
@@ -483,18 +531,49 @@ var vboxChooser = {
 	},
 	
 
+	/*
+	 * Returns true if there are VMs with ID vmid that are not selected
+	 */
+	vmHasUnselectedCopy : function (vmid) {
+		return ($(vboxChooser._anchor).find('table.vboxChooserItem-'+vboxChooser._anchorid+'-'+vmid+':not(.vboxListItemSelected)').length > 0);
+	},
+	
+	/*
+	 * Remove selected VMs from the list and rewrite group definitions
+	 * this assumes that there are other copies of these VMs that are not
+	 * selected.
+	 */
+	removeVMs : function(vmids) {
+		
+		for(var i = 0; i < vmids.length; i++) {
+			$(vboxChooser._anchor).find('table.vboxChooserItem-'+vboxChooser._anchorid+'-'+vmids[i]+'.vboxListItemSelected').remove();		
+		}
+				
+		// Update selection list
+		vboxChooser._selectedList = vboxChooser._selectedList.filter(function(v){
+			return (v.type == 'group' || (jQuery.inArray(v.id, vmids) == -1));
+		});
 
+		// Tell interface that selection list has changed
+		vboxChooser.selectionListChanged(vboxChooser._selectedList);
+
+		// compose and save group definitions
+		vboxChooser.composeGroupDef(true);
+		
+		
+		// Possible resize needed
+		vboxChooser._resizeElements(true);
+
+	},
+	
 	/*
 	 * Generate HTML from VM definition
 	 */
 	vmHTML : function (vmn) {
 		
 		var tbl = $('<table />').attr({'class':'vboxChooserItem-'+vboxChooser._anchorid+'-'+vmn.id + " vboxChooserVM"})
-			.bind('mousedown',vboxChooser.selectItem)
-			.hover(function(){
-				$(this).addClass('vboxHover');
-				},function(){$(this).removeClass('vboxHover');
-			}).data('vmid',vmn.id);
+			.on('mousedown',vboxChooser.selectItem)
+			.hoverClass('vboxHover').data('vmid',vmn.id);
 		
 		
 		// Drag-and-drop functionality
@@ -507,6 +586,8 @@ var vboxChooser = {
 			// drag start
 			},'start':function(e) {
 							
+				if(!vboxChooser._editable) return false;
+				
 				$(vboxChooser._anchor).disableSelection();
 				vboxChooser._dragging = vmn.id;
 				$(vboxChooser._anchor).find('table.vboxHover').removeClass('vboxHover');
@@ -570,13 +651,7 @@ var vboxChooser = {
 					},
 					function(a) {
 						
-						// Check for unfinished progress operations
-						if($('#vboxProgressOps').children('div:not(.vboxProgressComplete)').addClass('vboxProgressRunning').length) {
-							vboxAlert('Cannot change servers while operations are in progress');
-							return;
-						}
-						
-						if(a == $('#vboxPane').data('vboxConfig').name) return;						
+						if(a == $('#vboxPane').data('vboxConfig').name) return;				
 						
 						// Show loading screen
 						var l = new vboxLoader();
@@ -614,11 +689,13 @@ var vboxChooser = {
 		// Not rendering host
 		} else {
 			
-			$(td).append('<div class="vboxFitToContainer"><span class="vboxVMName">'+$('<span />').text(vmn.name).html()+'</span>'+ (vmn.currentSnapshotName ? ' (' + $('<span />').text(vmn.currentSnapshotName).html() + ')' : '')+'</div>');
+			$(td).append('<div class="vboxFitToContainer vboxVMName"><span class="vboxVMName">'+$('<span />').text(vmn.name).html()+'</span>'+ (vmn.currentSnapshotName ? '<span class="vboxVMChooserSnapshotName"> (' + $('<span />').text(vmn.currentSnapshotName).html() + ')</span>' : '')+'</div>');
 			
 
 			// Table gets tool tips
-			tip = vboxChooser._vmToolTip.replace('%1',('<b>'+$('<span />').text(vmn.name).html()+'</b>'+(vmn.currentSnapshotName ? ' (' + $('<span />').text(vmn.currentSnapshotName).html() + ')' : '')))
+			tip = trans(vboxChooser._vmToolTip, 'UIVMListView').replace('%1',('<b>'+$('<span />')
+				.text(vmn.name).html()+'</b>'+(vmn.currentSnapshotName ? ' (' + $('<span />')
+						.text(vmn.currentSnapshotName).html() + ')' : '')))
 				.replace('%2',trans(vboxVMStates.convert(vmn.state),'VBoxGlobal'))
 				.replace('%3',vboxDateTimeString(vmn.lastStateChange))
 				.replace('%4',trans(vmn.sessionState,'VBoxGlobal').toLowerCase());
@@ -630,12 +707,12 @@ var vboxChooser = {
 		
 		// VM state row
 		var tr = $('<tr />');
-		var td = $('<td />').attr({'class':(vmn.id != 'host' && vmn.sessionState != 'Unlocked' ? 'vboxVMSessionOpen' : '')}).html("<img src='images/vbox/" + vboxMachineStateIcon(vmn.state) +"' /><span class='vboxVMState'>" + trans(vboxVMStates.convert(vmn.state),'VBoxGlobal') + '</span>');
+		var td = $('<td />').attr({'class':(vmn.id != 'host' && vmn.sessionState != 'Unlocked' ? 'vboxVMSessionOpen' : '')});
 
 		// Add VirtualBox version if hosting
 		if(vmn.id == 'host') {
 			
-			$(td).append(' - ' + $('#vboxPane').data('vboxConfig').version.string);
+			$(td).html("<div class='vboxFitToContainer vboxVMState'><img src='images/vbox/" + vboxMachineStateIcon(vmn.state) +"' /><span class='vboxVMState'>" + trans(vboxVMStates.convert(vmn.state),'VBoxGlobal') + ' - ' + $('#vboxPane').data('vboxConfig').version.string+'</span></div>');
 			
 			// Check for version mismatches?
 			if(!vboxChooser._versionChecked) {
@@ -643,9 +720,11 @@ var vboxChooser = {
 				var vStr = $('#vboxPane').data('vboxConfig').phpvboxver.substring(0,$('#vboxPane').data('vboxConfig').phpvboxver.indexOf('-'));
 				var vers = $('#vboxPane').data('vboxConfig').version.string.replace('_OSE','').split('.');
 				if(vers[0]+'.'+vers[1] != vStr) {
-					vboxAlert('This version of phpVirtualBox ('+$('#vboxPane').data('vboxConfig').phpvboxver+') is incompatible with VirtualBox ' + $('#vboxPane').data('vboxConfig').version.string + ". You probably need to <a href='http://code.google.com/p/phpvirtualbox/downloads/list?q=phpvirtualbox-"+vers[0]+'.'+vers[1]+"' target=_blank>download the latest phpVirtualBox " + vers[0]+'.'+vers[1] + "-x</a>.<p>See the Versioning section <a href='http://code.google.com/p/phpvirtualbox/downloads/detail?name=README.txt' target=_blank>here</a> for more information</p>",{'width':'auto'});
+					vboxAlert('This version of phpVirtualBox ('+$('#vboxPane').data('vboxConfig').phpvboxver+') is incompatible with VirtualBox ' + $('#vboxPane').data('vboxConfig').version.string + ". You probably need to <a href='http://sourceforge.net/projects/phpvirtualbox/files/' target=_blank>download the latest phpVirtualBox " + vers[0]+'.'+vers[1] + "-x</a>.<p>See the Versioning section below the file list in the link for more information</p>",{'width':'auto'});
 				}
 			}			
+		} else {
+			$(td).html("<div class='vboxFitToContainer vboxVMState'><img src='images/vbox/" + vboxMachineStateIcon(vmn.state) +"' /><span class='vboxVMState'>" + trans(vboxVMStates.convert(vmn.state),'VBoxGlobal') + '</span></div>');
 		}
 		
 		$(tr).append(td).appendTo(tbl);
@@ -687,19 +766,22 @@ var vboxChooser = {
 		
 	},
 
+	
 	/*
 	 * VM Group Dropped
 	 */
 	vmGroupDropped : function(e, droppedGroup) {
 		
+		
 		$(vboxChooser._anchor).enableSelection();
+
 		
-		var vmGroupPath = vboxChooser._draggingGroup;
-		
+		var vmGroupPath = vboxChooser._draggingGroup;		
 		vboxChooser._draggingGroup = false;
-		
 		$(droppedGroup).removeClass('vboxHover');
 		
+		if(!vboxChooser._editable) return false;		
+		
 		// Cannot drag a group that contains a VM without
 		// an unlocked session state if it will modify VM
 		// Groups
@@ -834,9 +916,12 @@ var vboxChooser = {
 	 */
 	vmDropped : function (e, droppedVM){
 		
+		
 		$(vboxChooser._anchor).enableSelection();
 		vboxChooser._dragging = null;
 		
+		if(!vboxChooser._editable) return false;
+
 		// Cannot drag if this VM's session is not open
 		var thisSessionLocked = false;
 		var vmData = vboxVMDataMediator.getVMData($(droppedVM).data('vmid'));
@@ -962,7 +1047,6 @@ var vboxChooser = {
 					if(!$(dropTarget)[0])
 						dropTarget = vboxChooser._anchor.children('div.vboxChooserGroup');					
 
-				
 					// Already in this list?
 					if(dropTarget.children('div.vboxChooserGroupVMs').children('table.vboxChooserItem-'+vboxChooser._anchorid+'-'+$(droppedVM).data('vmid'))[0]) {
 						return true;
@@ -971,7 +1055,7 @@ var vboxChooser = {
 					if(!e.ctrlKey && !e.metaKey) {
 						$(droppedVM).detach().appendTo(dropTarget.children('div.vboxChooserGroupVMs').first());
 					} else {
-						vboxChooser.vmHTML(vmData).appendTo(dropTarget.children('div.vboxChooserGroup').children('div.vboxChooserGroupVMs').first());
+						vboxChooser.vmHTML(vmData).appendTo(dropTarget.children('div.vboxChooserGroupVMs').first());
 					}
 
 				}
@@ -1161,15 +1245,31 @@ var vboxChooser = {
 				$(elm).children('div.vboxChooserGroupVMs').css({'display':(vmList.length || $(elm).data('vmGroupPath') == '/' ? '' : 'none')})
 					.siblings('div.vboxChooserGroupHeader')
 					.each(function(hidx,header) {
-						$(header).tipped({'source':'<strong>'+$(header).siblings('div.vboxChooserGroupIdentifier').attr('title')+'</strong>'+
-								(gList.length ? ('<br />' + trans('%n group(s)','UIGChooserItemGroup',gList.length).replace('%n',gList.length)) : '') +
-								(vmList.length ? ('<br />' + trans('%n machine(s)','UIGChooserItemGroup',vmList.length).replace('%n',vmList.length)) : '')
+						
+						var staticTip = '<strong>'+$(header).siblings('div.vboxChooserGroupIdentifier').attr('title')+'</strong>'+
+							(gList.length ? ('<br />' + trans('%n group(s)','UIGChooserItemGroup',gList.length).replace('%n',gList.length)) : '') +
+							(vmList.length ? ('<br />' + trans('%n machine(s)','UIGChooserItemGroup',vmList.length).replace('%n',vmList.length)) : '');
+
+						$(header).tipped({'source':function() {
+									
+							// find number of running VMs
+							var runningVMs = 0;
+							
+							if(vmList.length) {
+								$(header).siblings('div.vboxChooserGroupVMs').find('td.vboxVMSessionOpen').each(function(idx,elm3) {
+									if(vboxVMStates.isRunning(vboxVMDataMediator.getVMData($(elm3).closest('table').data('vmid'))))
+										runningVMs++;
+								});								
+							}
+							
+							return staticTip + (runningVMs > 0 ? ' ' + trans('(%n running)','UIGChooserItemGroup',runningVMs).replace('%n', runningVMs) : '');
+						}
 							,'position':'mouse','delay':1500});
 					})
 						.children('span.vboxChooserGroupInfo')
 						.children('span.vboxChooserGroupCounts').html(
-							(gList.length ? ('<img src="images/vbox/nw_16px.png" />'+gList.length) : '') +
-							(vmList.length ? ('<img src="images/vbox/fullscreen_16px.png" />'+vmList.length) : '')
+							(gList.length ? ('<span style="background-image:url(images/vbox/group_abstract_16px.png);" />'+gList.length) : '') +
+							(vmList.length ? ('<span style="background-image:url(images/vbox/machine_abstract_16px.png);" />'+vmList.length) : '')
 				);
 			});
 			
@@ -1178,15 +1278,19 @@ var vboxChooser = {
 		// Save GUI group definition?
 		if(!save) return;
 		
+		
+		// Tell the interface we're about to save groups
+		vboxChooser._editable = false;
+		$('#vboxPane').trigger('vmGroupDefsSaving');
+		
 		vboxChooser._groupDefs = allGroups;
-		vboxAjaxRequest('vboxGroupDefinitionsSet',{'groupDefinitions':allGroups});		
 		
 		// Save machine groups and trigger change
 		var vms = [];
 		var vmList = vboxVMDataMediator.getVMList();
 		for(var i = 0; i < vmList.length; i++) {
 			
-			if(vmList[i].id == 'host') continue;
+			if(!vmList[i] || vmList[i].id == 'host') continue;
 
 			/* If a VM's groups have changed, add it to the list */
 			var eGroups = vmList[i].groups;
@@ -1195,8 +1299,8 @@ var vboxChooser = {
 			if($(nGroups).not(eGroups).length || $(eGroups).not(nGroups).length) {
 			
 				vms[vms.length] = {
-						'id' : vmList[i].id,
-						'groups' : nGroups
+					'id' : vmList[i].id,
+					'groups' : nGroups
 				};				
 			}
 		}
@@ -1230,10 +1334,22 @@ var vboxChooser = {
 				
 				if(res.responseData.errored) {
 					reloadAll();
+					vboxChooser._editable = true;
+					$('#vboxPane').trigger('vmGroupDefsSaved');
+				} else {
+					$.when(vboxAjaxRequest('vboxGroupDefinitionsSet',{'groupDefinitions':allGroups})).always(function(){
+						vboxChooser._editable = true;
+						$('#vboxPane').trigger('vmGroupDefsSaved');
+					});
 				}
 				
 			}).fail(reloadAll);
 			
+		} else {
+			$.when(vboxAjaxRequest('vboxGroupDefinitionsSet',{'groupDefinitions':allGroups})).always(function(){
+				vboxChooser._editable = true;
+				$('#vboxPane').trigger('vmGroupDefsSaved');
+			});
 		}
 		
 		
@@ -1467,9 +1583,9 @@ var vboxChooser = {
 		$(el).children('div.vboxChooserGroupHeader').append(
 			
 			$('<form />').append(
-				$('<input />').attr({'type':'text','value':$(el).children('div.vboxChooserGroupIdentifier').attr('title')}).css({'width':'90%','padding':'0px','margin':'0px'}).bind('keypress',function(e){
+				$('<input />').attr({'type':'text','value':$(el).children('div.vboxChooserGroupIdentifier').attr('title')}).css({'width':'90%','padding':'0px','margin':'0px'}).on('keypress',function(e){
 					if (e.which == 13) {
-						$(this).unbind('blur', renameGroup);
+						$(this).off('blur', renameGroup);
 						renameGroup(e,this);
 						e.stopPropagation();
 						e.preventDefault();
@@ -1485,59 +1601,17 @@ var vboxChooser = {
 	},
 
 	/*
-	 * Select a single group and modify canvas elements
+	 * Select a single group
 	 */
-	_selectGroup : function(gelm) {
-		
-		if(isCanvasSupported()) {
-			
-			// Change collapse arrow color
-			$(gelm).children('div.vboxChooserGroupHeader')			
-				.children('canvas.vboxChooserGroupNameArrowCollapse').each(function(idx,can) {
-					var ctx = $(can)[0].getContext('2d');
-					ctx.clearRect(0,0,18,18);
-					if($(gelm).hasClass('vboxVMGroupCollapsed')) {
-						ctx.save();
-						ctx.translate(9,9);
-						ctx.rotate(-90 * Math.PI / 180.0);										
-						ctx.drawImage(vboxImageDownWhite,-9,-9,18,18);
-						ctx.restore();
-					} else {
-						ctx.drawImage(vboxImageDownWhite,0,0,18,18);
-					}
-					
-				});
-		}
-		
+	_selectGroup : function(gelm) {		
 		$(gelm).addClass('vboxVMGroupSelected');
 	},
 	
 	/*
-	 * Deselect a single group and modify canvas elements
+	 * Deselect a single group
 	 */
 	_deselectGroup : function(gelm) {
 		
-		if(isCanvasSupported()) {
-			
-			// Change collapse arrow color
-			$(gelm).children('div.vboxChooserGroupHeader')			
-				.children('canvas.vboxChooserGroupNameArrowCollapse').each(function(idx,can) {
-		
-					var ctx = $(can)[0].getContext('2d');
-					ctx.clearRect(0,0,18,18);
-					if($(can).closest('div.vboxChooserGroup').hasClass('vboxVMGroupCollapsed')) {
-						ctx.save();
-						ctx.translate(9,9);
-						ctx.rotate(-90 * Math.PI / 180.0);										
-						ctx.drawImage(vboxImageDownGrey,-9,-9,18,18);
-						ctx.restore();
-					} else {
-						ctx.drawImage(vboxImageDownGrey,0,0,18,18);
-				}
-			
-			});
-		}
-		
 		$(gelm).removeClass('vboxVMGroupSelected');
 	},
 	
@@ -1744,7 +1818,7 @@ var vboxChooser = {
 				$(gelm).addClass('vboxChooserGroupShowOnly vboxChooserGroupRootLevel').siblings().addClass('vboxChooserGroupHide');
 
 				$.when(vboxChooser._anchor.show('slide', {direction: (back ? 'left' : 'right'), distance: (vboxChooser._anchor.outerWidth()/1.5)}, 200))
-					.then(function(){
+					.done(function(){
 						
 						// Restore scrolling
 						vboxChooser._anchor.css({'overflow-y':'auto'});
@@ -1756,7 +1830,7 @@ var vboxChooser = {
 						vboxChooser._resizeElements();
 						
 						// force redraw of these
-						$(gelm).find('.vboxFitToContainer').css({'display':'none'}).css({'width':'','display':'inline-block'});
+						$(gelm).find('.vboxFitToContainer').css({'display':'none'}).css({'width':'','display':''});
 						
 
 					});
@@ -1780,7 +1854,7 @@ var vboxChooser = {
 				vboxChooser._anchor.children('div.vboxChooserGroupRoot').addClass('vboxChooserGroupRootLevel');
 
 				$.when(vboxChooser._anchor.show('slide', {direction: 'left', distance: (vboxChooser._anchor.outerWidth()/1.5)}, 200))
-					.then(function(){
+					.done(function(){
 
 						// Restore scrolling
 						vboxChooser._anchor.css({'overflow-y':'auto'});
@@ -1792,7 +1866,7 @@ var vboxChooser = {
 						vboxChooser._resizeElements();
 
 						// force redraw of these
-						vboxChooser._anchor.find('.vboxFitToContainer').css({'display':'none','width':''}).css({'display':'inline-block'});
+						vboxChooser._anchor.find('.vboxFitToContainer').css({'display':'none','width':''}).css({'display':''});
 					});
 			});
 		}
@@ -1821,65 +1895,37 @@ var vboxChooser = {
 					
 					// Button rotation function
 					var rotateButton = function(){return true;};
-					
-					// Canvas rotation
-					if(isCanvasSupported()) {
 						
-						var ctx = $(this).children('canvas.vboxChooserGroupNameArrowCollapse')[0].getContext('2d');
-						
-						var imgButton = ($(this).closest('div.vboxChooserGroup').hasClass('vboxVMGroupSelected') ?
-								vboxImageDownWhite : vboxImageDownGrey);
+					var vboxArrowImage = $(this).find('span.vboxChooserGroupNameArrowCollapse');
 
+					if(!($.browser.msie && $.browser.version.substring(0,1) < 9)) {
+						
 						rotateButton = function() {
 							
-							return $('<div />').animate({left:90},
-									{
+							return $('<div />').animate({left:90},{
 								duration: 300,
-								step: function(currentStep ) {
-									ctx.save();
-									ctx.clearRect(0,0,18,18);
-									ctx.translate(9,9);
-									ctx.rotate((collapsed ? -90 + currentStep : (currentStep*-1)) * Math.PI / 180.0);
-									ctx.drawImage(imgButton,-9,-9,18,18);
-									ctx.restore();
-								},
-								queue: true
-							});							
-						};
-						
-					// Span rotation
-					} else if(!($.browser.msie && $.browser.version.substring(0,1) < 9)){
-						
-						var vboxArrowImage = $(this).find('span.vboxChooserGroupNameArrowCollapse');
-
-						rotateButton = function() {
-						
-							return $('<div />').animate({left:90},
-									{
-										duration: 300,
-										step: function(currentStep ) {
-											currentStep *= -1;
-											if(collapsed) currentStep = (-90 + -currentStep);
-											vboxArrowImage.css({
-												'transform':'rotate('+currentStep+'deg)',
-												'-moz-transform': 'rotate('+currentStep+'deg)',
-												'-webkit-transform': 'rotate('+currentStep+'deg)',
-												'-o-transform': 'rotate('+currentStep+'deg)',
-												'-ms-transform': 'rotate('+currentStep+'deg)'
-											});
-										},
-										queue: true,
-										complete: function() {
-											vboxArrowImage.css({
-												'transform':'',
-												'-moz-transform': '',
-												'-webkit-transform': '',
-												'-o-transform': '',
-												'-ms-transform': ''
-											});
-										}
-									
+								step: function(currentStep) {
+									if(!collapsed) currentStep = (90 - currentStep);
+									vboxArrowImage.css({
+										'transform':'rotate('+currentStep+'deg)',
+										'-moz-transform': 'rotate('+currentStep+'deg)',
+										'-webkit-transform': 'rotate('+currentStep+'deg)',
+										'-o-transform': 'rotate('+currentStep+'deg)',
+										'-ms-transform': 'rotate('+currentStep+'deg)'
 									});
+								},
+								queue: true,
+								complete: function() {
+									vboxArrowImage.css({
+										'transform':'',
+										'-moz-transform': '',
+										'-webkit-transform': '',
+										'-o-transform': '',
+										'-ms-transform': ''
+									});
+								}
+								
+							});
 						};
 					}
 
@@ -1906,29 +1952,7 @@ var vboxChooser = {
 						})
 				)
 				.append(
-						(isCanvasSupported() ? 
-							$('<canvas />').addClass('vboxChooserGroupNameArrowLeft vboxChooserGroupNameArrowCollapse')
-								.attr({'width':'18','height':'18','style':'width: 18px; height: 18px;'})
-								.each(function(idx, canvas) {
-									var ctx = canvas.getContext('2d');
-									if(collapsed) {
-										ctx.save();
-										ctx.translate(9,9);
-										ctx.rotate(-90 * Math.PI / 180.0);										
-										ctx.drawImage(vboxImageDownGrey,-9,-9,18,18);
-										ctx.restore();
-									} else {
-										ctx.drawImage(vboxImageDownGrey,0,0,18,18);
-									}
-								}).mousedown(function(e){
-									e.stopPropagation();
-									e.preventDefault();
-									return false;
-								}).mouseup(function(){
-									$(this).closest('div.vboxChooserGroupHeader').trigger('dblclick');									
-								})
-								
-							: $('<span />').addClass('vboxChooserGroupNameArrowLeft vboxChooserGroupNameArrowCollapse vboxArrowImage')
+						$('<span />').addClass('vboxChooserGroupNameArrowLeft vboxChooserGroupNameArrowCollapse vboxArrowImage')
 								.mousedown(function(e){
 									e.stopPropagation();
 									e.preventDefault();
@@ -1936,7 +1960,7 @@ var vboxChooser = {
 								}).mouseup(function(){
 									$(this).closest('div.vboxChooserGroupHeader').trigger('dblclick');									
 								})
-						)
+						
 				).append(
 						
 					$('<span />').addClass('vboxChooserGroupNameArrowLeft vboxChooserGroupShowOnlyBack vboxArrowImage')
@@ -1974,6 +1998,8 @@ var vboxChooser = {
 				)
 				.hover(function(){
 					
+					if(vboxChooser._compact) return;
+					
 					$(this).addClass('vboxHover');
 
 					// Resize title and add hover class?
@@ -1996,7 +2022,7 @@ var vboxChooser = {
 					if(!$(this).parent().hasClass('vboxChooserGroupRoot'))
 						$(this).children('span.vboxChooserGroupName').css({'max-width':''});
 					
-				}).bind('mousedown',vboxChooser.selectItem)				
+				}).on('mousedown',vboxChooser.selectItem)				
 
 			).addClass((first ? 'vboxChooserGroupRoot vboxChooserGroupRootLevel ' : (collapsed ? 'vboxVMGroupCollapsed ' : '')) + 'vboxChooserGroup')
 			.data({'vmGroupPath':gpath})
@@ -2011,6 +2037,8 @@ var vboxChooser = {
 										
 					if(!$('#vboxPane').data('vboxSession').admin) return false;
 					
+					if(!vboxChooser._editable) return false;
+					
 					vboxChooser._draggingGroup = $(this).data('vmGroupPath');
 					$(vboxChooser._anchor).disableSelection();
 				
@@ -2081,9 +2109,10 @@ var vboxChooser = {
 		// Where are we drawn?
 		if(anchorid) {
 			vboxChooser._anchorid = anchorid;
-			vboxChooser._anchor = $('#'+anchorid);			
+			vboxChooser._anchor = $('#'+anchorid);
 		}
 		
+		
 		// Set group definition key
 		vboxChooser._groupDefinitionKey = $('#vboxPane').data('vboxConfig')['groupDefinitionKey'];
 		
@@ -2133,17 +2162,26 @@ $(document).ready(function(){
 	vboxChooser._scrollbarWidth = getScrollbarWidth();
 	
 	// "Stop" chooser
-	$('#vboxPane').bind('hostChange',function(){
+	$('#vboxPane').on('hostChange',function(){
 	
 		vboxChooser.stop();
 		
-	}).bind('hostChanged',function(){
+	}).on('hostChanged',function(){
 	
 	
 		vboxChooser.start();
 		
+	// Refresh menus
+	}).on('vmGroupDefsSaving vmGroupDefsSaved vmSelectionListChanged', function() {
+		
+		if(vboxChooser._vmGroupContextMenuObj)
+			vboxChooser._vmGroupContextMenuObj.update(vboxChooser);		
+		if(vboxChooser._vmContextMenuObj)
+			vboxChooser._vmContextMenuObj.update(vboxChooser);
+
+			
 	// Event list queue
-	}).bind('vboxEvents',function(e, eventList) {
+	}).on('vboxEvents',function(e, eventList) {
 
 		var redrawVMs = [];
 		var sortGroups = [];
@@ -2176,6 +2214,9 @@ $(document).ready(function(){
 
 						redrawVMs[redrawVMs.length] = vmid;
 						
+						// Make sure VM has root group at least
+						if(data.groups.length == 0) data.groups = ['/'];
+						
 						// Remove from groups if they have changed
 						var currGroups  = vboxChooser.getGroupsForVM(vmid);
 						var groupDiff = $(currGroups).not(data.groups);
@@ -2204,9 +2245,9 @@ $(document).ready(function(){
 								continue;
 							
 							$(gElm).children('div.vboxChooserGroupVMs')
-							.append(								
-									vboxChooser.vmHTML(data)
-							);
+								.append(								
+										vboxChooser.vmHTML(data)
+								);
 							
 							selectedChanged = (selectedChanged || $(gElm).children('div.vboxChooserGroupVMs').closest('div.vboxVMGroupSelected').length);
 							
@@ -2224,10 +2265,13 @@ $(document).ready(function(){
 					
 				/////////////////////////////////
 				//
-				// Snapshot taken
+				// Snapshot taken / deleted / restored
 				//
 				/////////////////////////////////
+				case 'OnSnapshotDeleted':
 				case 'OnSnapshotTaken':
+				case 'OnSnapshotRestored':
+				case 'OnSnapshotChanged':
 					redrawVMs[redrawVMs.length] = eventList[i].machineId;
 					break;
 				
@@ -2291,7 +2335,7 @@ $(document).ready(function(){
 				case 'OnExtraDataChanged':
 					
 					if(!eventList[i].machineId && eventList[i].key.indexOf(vboxChooser._groupDefinitionKey) === 0) {
-						
+												
 						var path = eventList[i].key.substring(vboxChooser._groupDefinitionKey.length);
 						if(!path) path = "/";
 						var name = path.substring(path.lastIndexOf('/')+1);
@@ -2400,20 +2444,8 @@ $(document).ready(function(){
 
 		
 
-	// Update menus on selection list change
-	}).bind('vmSelectionListChanged',function(){
-		
-		if(vboxChooser._vmGroupContextMenuObj)
-			vboxChooser._vmGroupContextMenuObj.update(vboxChooser);
-	
-		
-		if(vboxChooser._vmContextMenuObj)
-			vboxChooser._vmContextMenuObj.update(vboxChooser);
-		
-	
 	});
 	
 	
-	
 
 });
\ No newline at end of file
